@@ -5,10 +5,11 @@ use warnings;
 
 use Carp;
 use Encode;
-use JSON;
+use JSON::MaybeXS;
 use HTTP::Request;
 use LWP::UserAgent;
 use LWP::Protocol::https;
+use Params::Get;
 use URI;
 
 =head1 NAME
@@ -17,11 +18,11 @@ Geo::Coder::XYZ - Provides a Geo-Coding functionality using L<https://geocode.xy
 
 =head1 VERSION
 
-Version 0.09
+Version 0.10
 
 =cut
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 =head1 SYNOPSIS
 
@@ -46,13 +47,26 @@ Geo::Coder::XYZ provides an interface to geocode.xyz, a free Geo-Coding database
 =cut
 
 sub new {
-	my($class, %param) = @_;
+	my $proto = shift;
+	my $class = ref($proto) || $proto;
 
-	my $ua = delete $param{ua} || LWP::UserAgent->new(agent => __PACKAGE__ . "/$VERSION");
-	if(!defined($param{'host'})) {
+	# Use Geo::Coder::XYZ->new(), not Geo::Coder::XYZ::new()
+	if(!defined($class)) {
+		carp(__PACKAGE__, ' use ->new() not ::new() to instantiate');
+		return;
+	}
+
+	my $params = Params::Get::get_params(undef, \@_);
+
+	my $ua = $params->{ua};
+	if(!defined($ua)) {
+		$ua = LWP::UserAgent->new(agent => __PACKAGE__ . "/$VERSION");
+		$ua->default_header(accept_encoding => 'gzip,deflate');
+	}
+	if(!defined($params->{'host'})) {
 		$ua->ssl_opts(verify_hostname => 0);	# Yuck
 	}
-	my $host = delete $param{host} || 'geocode.xyz';
+	my $host = $params->{host} || 'geocode.xyz';
 
 	return bless { ua => $ua, host => $host }, $class;
 }
@@ -65,27 +79,23 @@ sub new {
     print 'Longitude: ', $location->{'longt'}, "\n";
 
     @locations = $geo_coder->geocode('Portland, USA');
-    diag 'There are Portlands in ', join (', ', map { $_->{'state'} } @locations);
+    print 'There are Portlands in ', join (', ', map { $_->{'state'} } @locations), "\n";
 
 =cut
 
 sub geocode {
 	my $self = shift;
-	my %param;
+	my $params = Params::Get::get_params('location', \@_);
 
-	if(ref($_[0]) eq 'HASH') {
-		%param = %{$_[0]};
-	} elsif(ref($_[0])) {
-		Carp::croak('Usage: geocode(location => $location)');
-	} elsif(@_ % 2 == 0) {
-		%param = @_;
-	} else {
-		$param{location} = shift;
-	}
-
-	my $location = $param{location}
+	my $location = $params->{location}
 		or Carp::croak('Usage: geocode(location => $location)');
 
+	# Fail when the input is just a set of numbers
+	if($params->{'location'} !~ /\D/) {
+		Carp::croak('Usage: ', __PACKAGE__, ": invalid input to geocode(), $params->{location}");
+		return;
+	}
+
 	if (Encode::is_utf8($location)) {
 		$location = Encode::encode_utf8($location);
 	}
@@ -110,10 +120,10 @@ sub geocode {
 		return { };
 	}
 
-	my $json = JSON->new()->utf8();
+	my $json = JSON::MaybeXS->new()->utf8();
 	my $rc;
 	eval {
-		$rc = $json->decode($res->content());
+		$rc = $json->decode($res->decoded_content());
 	};
 	if(!defined($rc)) {
 		if($@) {
@@ -127,7 +137,7 @@ sub geocode {
 	if($rc->{'otherlocations'} && $rc->{'otherlocations'}->{'loc'} &&
 	   (ref($rc->{'otherlocations'}->{'loc'}) eq 'ARRAY')) {
 		my @rc = @{$rc->{'otherlocations'}->{'loc'}};
-	   	if(wantarray) {
+		if(wantarray) {
 			return @rc;
 		}
 		return $rc[0];
@@ -149,7 +159,9 @@ environment variables:
 You can also set your own User-Agent object:
 
     use LWP::UserAgent::Throttled;
-    $geo_coder->ua(LWP::UserAgent::Throttled->new());
+    my $ua = LWP::UserAgent::Throttled->new();
+    $ua->throttle({ 'geocode.xyz' => 2 });
+    $geo_coder->ua($ua);
 
 =cut
 
@@ -169,25 +181,24 @@ Similar to geocode except it expects a latitude/longitude parameter.
 
 =cut
 
-sub reverse_geocode {
+sub reverse_geocode
+{
 	my $self = shift;
+	my $params = Params::Get::get_params('latlng', \@_);
 
-	my %param;
-	if (@_ % 2 == 0) {
-		%param = @_;
-	} else {
-		$param{latlng} = shift;
-	}
-
-	my $latlng = $param{latlng}
+	my $latlng = $params->{'latlng'}
 		or Carp::carp('Usage: reverse_geocode(latlng => $latlng)');
 
 	return $self->geocode(location => $latlng, reverse => 1);
 }
 
+=head1 SUPPORT
+
+This module is provided as-is without any warranty.
+
 =head1 AUTHOR
 
-Nigel Horne <njh@bandsman.co.uk>
+Nigel Horne, C<< <njh at nigelhorne.com> >>
 
 Based on L<Geo::Coder::GooglePlaces>.
 
@@ -202,7 +213,7 @@ L<Geo::Coder::GooglePlaces>, L<HTML::GoogleMaps::V3>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2017-2018 Nigel Horne.
+Copyright 2017-2025 Nigel Horne.
 
 This program is released under the following licence: GPL2
 
